// based on Tom Gallacher's code. https://github.com/tomgco/Arduino_Helicopter
// modified by 
// Thomas Buchberger and David Thaller 

#include <avr/io.h>
#include <avr/interrupt.h>
#define LED 8
#define STATUS 13
         
void setup() {
	pinMode(LED, OUTPUT); // setup pins
	digitalWrite(LED, LOW);
	pinMode(STATUS, OUTPUT); // setup pins
	digitalWrite(STATUS, LOW);
	Serial.begin(600); // setup serial communication we will start at 600 baud
        byte backup[] = {0,0,0,0,0};
}
//sends 38Khz pulse when using a 16Mhz ic
void sendPulse(long us) {
	for(int i = 0; i < (us / 26) - 1; i++) {
		 digitalWrite(LED, HIGH);
		 delayMicroseconds(10);
		 digitalWrite(LED, LOW);
		 delayMicroseconds(10);
	}
}
void sendHeader() {
	// Start 38Khz pulse for 2000us
	sendPulse(2002);
	// 2000us off.
	delayMicroseconds(1998);
}
void sendFooter() {
	sendPulse(312);
	delay(60);
}

/*
* Start of transmission is a 2000us on, 2000us off.
*/
void sendControlPacket(byte channel, byte yaw, byte pitch, byte throttle, byte trim) {
	static byte dataPointer, maskPointer;
	static const byte mask[] = {1, 2, 4, 8, 16, 32, 64, 128};
	static byte data[4];
	// Control bytes.
	data[0] = yaw - 1; // 0 -> 127 where 63 is the mid point.
	data[1] = pitch - 1; // ditto
	data[2] = throttle -1; // Channel 1 = 0 -> 127 & Channel 2 = 0 -> 127
	data[3] = trim - 1;
	dataPointer = 4;
	maskPointer = 8;
	if (channel == 0x32) {
		data[2] = data[2] + 128; //as channel 2 is 128 -> 255
	}
	sendHeader();
	while(dataPointer > 0) {
		sendPulse(312);	

		if(data[4 - dataPointer] & mask[--maskPointer]) {
			delayMicroseconds(688); // send 1 - possibly 600
		} else {
			delayMicroseconds(288); // send 0 - possibly 300
		}

		if(!maskPointer) {
			maskPointer = 8; // reset mask
			dataPointer--; // decrement pointer in data byte array
		}
	}
	sendFooter();
}

byte channel = 0;
byte yaw = 0;
byte pitch = 0;
byte throttle = 0;
byte trim = 0;
byte backup[] = {0,0,0,0,0};
  
void loop() {
    if (Serial.available() > 6 && Serial.read() == 140 && Serial.read() == 143) {
         yaw = Serial.read();
         pitch = Serial.read();
         throttle = Serial.read();
         trim = Serial.read();
        
         if(Serial.read() == 255) {
           backup[0] = channel;
           backup[1] = yaw;
           backup[2] = pitch;
           backup[3] = throttle;
           backup[4] = trim;
           sendControlPacket(backup[0],backup[1],backup[2],backup[3],backup[4]); //send packet
           digitalWrite(STATUS,HIGH);            
           Serial.flush();
         }
         else { 
           digitalWrite(STATUS,LOW);
         }
    }
}
    

